#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <malloc.h>
#include <assert.h>
#include <servicelog-1/servicelog.h>

char *prog = NULL;

static void
usage(void)
{
	fprintf(stderr, "usage: [-v] %s nevents\n", prog);
	exit(1);
}

void
v1_api_error(servicelog *slog, int error_code, const char *message)
{
	fprintf(stderr, "%s: %d: %s\n", message, error_code,
					servicelog_error(slog));
}

static int
log_os_event(servicelog *slog, uint64_t *id, const char *message, int n,
	int severity, int serviceable)
{
	struct sl_event ev;
	struct sl_data_os os;
	struct sl_callout co;

	// char *serial = "TEMPmsrl";
	// char *model = "TEMP Machine Model; Fix retrieval";
	// char *model = "TEMPmdl";
	int rc;

	memset(&ev, 0, sizeof(ev));
	memset(&os, 0, sizeof(os));
	memset(&co, 0, sizeof(co));

	ev.time_event = time(NULL);
	ev.type = SL_TYPE_OS;
	ev.severity = severity;
	ev.refcode = "testOSev";

	ev.description = malloc(20 + strlen(message));
	assert(ev.description);
	sprintf(ev.description, "%d: %s", n, message);

	ev.machine_model = NULL;
	ev.machine_serial = NULL;

	ev.serviceable = serviceable;
	ev.predictive = !serviceable;
	ev.disposition = SL_DISP_RECOVERABLE;
	ev.call_home_status = SL_CALLHOME_NONE;
	ev.closed = 0;
	ev.callouts = &co;
	ev.raw_data_len = 0;
	ev.raw_data = NULL;
	ev.addl_data = &os;

	os.subsystem = "test_subsystem";
	os.driver = "test_driver";
	os.device = "test_device";

	co.next = NULL;
	co.priority = 'M';
	co.type = 42;
	// co.procedure = "Chill out";
	// co.location = "Pasadena";
	co.procedure = "FSPSP24";
	co.location = "U787E.001.0654321-P1";
	co.fru = "TEMPfru";
	co.serial = "TEMPcsrl";
	co.ccin = "TEMPccin";

	rc = servicelog_event_log(slog, &ev, id);
	if (rc)
		v1_api_error(slog, rc, __FUNCTION__);
	free(ev.description);
	// free(ev.machine_model);
	return rc;
}

main(int argc, char** argv)
{
	int i, nevents, rc, verbose = 0;
	servicelog *slog;
	uint64_t id;

	prog = argv[0];
	switch (argc) {
	case 2:
		nevents = atoi(argv[1]);
		break;
	case 3:
		if (strcmp(argv[1], "-v") != 0)
			usage();
		verbose++;
		nevents = atoi(argv[2]);
		break;
	default:
		usage();
	}

	rc = servicelog_open(&slog, SL_FLAG_ADMIN);
	if (rc) {
		fprintf(stderr, "servicelog_open returned %d\n", rc);
		exit(1);
	}

	for (i = 1; i <= nevents; i++) {
		rc = log_os_event(slog, &id, "dummy OS event", i,
						SL_SEV_ERROR_LOCAL, 1);
		if (rc != 0) {
			v1_api_error(slog, rc, "log_os_event");
			exit(2);
		}
		if (verbose)
			printf("event #%d: id = %lld\n", i, id);
	}

	servicelog_close(slog);
	exit(0);
}
