#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <malloc.h>
#include <assert.h>
#include <servicelog-1/servicelog.h>

char *prog = NULL;

static void
usage(void)
{
	fprintf(stderr, "usage: [-v] %s nevents\n", prog);
	exit(1);
}

static int
log_os_event(servicelog *slog, uint64_t *id, const char *message, int n,
	int severity, int serviceable)
{
	struct sl_event ev;
	struct sl_data_os os;
	char *model = "TEMP Machine Model; Fix retrieval";
	int rc;

	ev.time_event = time(NULL);
	ev.type = SL_TYPE_OS;
	ev.severity = severity;
	ev.refcode = "testOSev";

	ev.description = malloc(20 + strlen(message));
	assert(ev.description);
	sprintf(ev.description, "%d: %s", n, message);

	ev.machine_model = malloc(strlen(model));
	strcpy(ev.machine_model, model);

	ev.serviceable = serviceable;
	ev.predictive = !serviceable;
	ev.disposition = SL_DISP_RECOVERABLE;
	ev.call_home_status = SL_CALLHOME_NONE;
	ev.closed = 0;
	ev.callouts = NULL;
	ev.raw_data_len = 0;
	ev.raw_data = NULL;
	ev.addl_data = &os;

	os.subsystem = "test_subsystem";
	os.driver = "test_driver";
	os.device = "test_device";
printf("Calling servicelog_event_log()\n");

	rc = servicelog_event_log(slog, &ev, id);
	if (rc)
		printf("Error adding event through log.  Error num: %d\n", rc);
	else
		printf("Event registration successful\n");

	free(ev.description);
	return rc;
}

main(int argc, char** argv)
{
	int i, nevents, rc, verbose = 0;
	servicelog *slog;
	uint64_t id;

	rc = servicelog_open(&slog, SL_FLAG_ADMIN);
	if (rc) {
		fprintf(stderr, "servicelog_open returned %d\n", rc);
		exit(1);
	}

	printf("Attempting log\n");
	rc = log_os_event(slog, &id, "dummy OS event", i,
					SL_SEV_ERROR_LOCAL, 1);
	printf("Done with log\n");
	if (rc != 0) {

		rc = log_os_event(slog, &id, "dummy OS event", i,
						SL_SEV_ERROR_LOCAL, 1);
		if (rc != 0) {
			printf("Error adding event through log.  Error num: %d\n", rc);
			exit(2);
		}
		printf("event #%d: id = %lld\n", i, id);
		printf("Error adding event through log.  Error num: %d\n", rc);
		exit(2);
	}

	if (verbose)
		printf("event #%d: id = %lld\n", i, id);

	servicelog_close(slog);
	exit(0);
}
