#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <malloc.h>
#include <assert.h>
#include <unistd.h>
#include <servicelog-1/servicelog.h>

static char *prog = NULL;
static int split = 0, noloc = 0, verbose = 0;

static void
usage(void)
{
	fprintf(stderr, "usage: [-l] [-s] [-v] %s nevents\n", prog);
	exit(1);
}

static void
v1_api_error(servicelog *slog, int error_code, const char *message)
{
	fprintf(stderr, "%s: %d: %s\n", message, error_code,
					servicelog_error(slog));
}

static int
log_rtas_event(servicelog *slog, uint64_t *id, const char *message, int n,
	int severity, int serviceable)
{
	struct sl_event ev;
	struct sl_data_rtas rtas;
	struct sl_callout co, co2;

	int rc;
	int i;

	memset(&ev, 0, sizeof(ev));
	memset(&rtas, 0, sizeof(rtas));
	memset(&co, 0, sizeof(co));

	ev.time_event = time(NULL);
	ev.type = SL_TYPE_RTAS;
	ev.severity = severity;
	ev.refcode = "RTASsrc";

	ev.description = malloc(20 + strlen(message));
	assert(ev.description);
	sprintf(ev.description, "%d: %s", n, message);

	// Let libservicelog fill these in.  They're already NULL, btw.
	ev.machine_model = NULL;
	ev.machine_serial = NULL;

	ev.serviceable = serviceable;
	ev.predictive = !serviceable;
	ev.disposition = SL_DISP_RECOVERABLE;
	ev.call_home_status = SL_CALLHOME_NONE;
	ev.closed = 0;
	ev.callouts = &co;

	// This what gets passed to parse_rtas_event().  Too hard to fake
	// for now.
	ev.raw_data_len = 0;
	ev.raw_data = NULL;

	ev.addl_data = &rtas;

	rtas.action_flags = 0x8000;	// RTAS_UH_ACTION_SERVICE
	rtas.platform_id = 123;		// rtas_priv_hdr_scn.plid
	rtas.creator_id = 'E';		// RTAS_PH_CREAT_SERVICE_PROC
	rtas.subsystem_id = 0x30;	// rtas_usr_hdr_scn.subsystem_id
	rtas.pel_severity = 4;		// rtas_usr_hdr_scn.event_severity
	rtas.event_type = 3;		// RTAS_HDR_TYPE_INTERN_DEV_FAIL
	rtas.event_subtype = 0x10;	// RTAS_UH_TYPE_PREVIOUSLY_REPORTED
	rtas.kernel_id = 77;
	for (i = 0; i <= 7; i++)
		rtas.addl_words[i] = i + 0x10*i;
	
	co.next = NULL;
	co.priority = 'H';
	co.type = 43;
	// co.procedure = "Call IBM";
	// co.location = "RTasmania";
	co.procedure = "FSPSP24";
	if (split || noloc)
		co.location = NULL;
	else
		co.location = "U787E.001.0654321-P1";
	co.fru = "RTASfru";
	co.serial = "RTAScsrl";
	co.ccin = "RTASccin";

	if (split) {
		// Split procedure and location into different callouts.
		co.next = &co2;

		co2.next = NULL;
		co2.priority = 'M';
		co2.type = 44;
		co2.procedure = "FSPSP24";
		if (noloc)
			co2.location = NULL;
		else
			co2.location = "U787E.001.0654321-P1";
		co2.fru = "RTASfru";
		co2.serial = "RTAScsrl";
		co2.ccin = "RTASccin";
	}

	rc = servicelog_event_log(slog, &ev, id);
	if (rc)
		v1_api_error(slog, rc, __FUNCTION__);
	free(ev.description);
	// free(ev.machine_model);
	return rc;
}

main(int argc, char** argv)
{
	int i, nevents, rc;
	servicelog *slog;
	uint64_t id;
	int c;

	prog = argv[0];
	opterr = 0;
	while ((c = getopt(argc, argv, "lsv")) != -1) {
		switch (c) {
		case 'l':
			noloc = 1;
			break;
		case 's':
			split = 1;
			break;
		case 'v':
			verbose = 1;
			break;
		case '?':
			usage();
		}
	}
	if (optind != argc-1)
		usage();
	nevents = atoi(argv[optind]);

	rc = servicelog_open(&slog, SL_FLAG_ADMIN);
	if (rc) {
		fprintf(stderr, "servicelog_open returned %d\n", rc);
		exit(1);
	}

	for (i = 1; i <= nevents; i++) {
		rc = log_rtas_event(slog, &id, "dummy RTAS event", i,
						SL_SEV_ERROR_LOCAL, 1);
		if (rc != 0) {
			v1_api_error(slog, rc, "log_rtas_event");
			exit(2);
		}
		if (verbose)
			printf("event #%d: id = %lld\n", i, id);
	}

	servicelog_close(slog);
	exit(0);
}
