#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <malloc.h>
#include <assert.h>
#include <servicelog-1/libservicelog.h>
#include <getopt.h>

char *prog = NULL;

#define ARG_LIST	"i:h"

static struct option long_options[] = {
	{"id",		required_argument,		NULL, 'i'},
	{"help",	no_argument,        	NULL, 'h'},
	{0,0,0,0}
};

static void
usage(void)
{
	fprintf(stderr, "This is a test driver meant to grab an rtas event, specified by id, \n"
			"then exercise the parse_rtas_event() function is librtas. \n"
			" Meant to ensure we are returning properly formatted v29 event structs\n"
			"---------------------------------------------------------------\n", prog);
	fprintf(stderr, "Usage: -i 'id of event to attempt parse'\n", prog);
	fprintf(stderr, "-h : This message\n", prog);
	exit(1);
}

int
main(int argc, char *argv[])
{
	int option_index;
	int event_id = 0;
	struct servicelog slog;
	struct rtas_event *evt;
	char *buf;
	int rc = 0;
	size_t sz;
	void *data;

	for (;;) {
		option_index = 0;
		rc = getopt_long(argc, argv, ARG_LIST, long_options,
				 &option_index);

		if (rc == -1)
			break;

		switch (rc) {
		case 'i':
			event_id = atoi(optarg);
			break;
		case 'h':
			usage();
			break;
		default:
			printf("Invalid argument\n");
		}
	}

	if (!event_id) {
		printf("Event id invalid: %d\n", event_id);
		exit(-1);
	}

	rc = servicelog_open(&slog, "", 0);
	if (rc != 0) {
		fprintf(stderr, "Error opening servicelog database: %s\n", strerror(rc));
		return -1;
	}

	rc = servicelog_get_event(&slog, event_id, &data, &sz);
	if (rc != 0) {
			fprintf(stderr, "%s\n", servicelog_error(&slog));
			servicelog_close(&slog);
			return 2;
	}

	printf("Rtas event:\n");
	servicelog_print_event(stdout, data, 1);
	printf("------------------------------------------------------\n"
			" Rtas event header:\n");
	servicelog_print_header(stdout, data, 0);

	printf("\n\n Attempting to parse this rtas event:\n");
	evt = parse_rtas_event(data, sz);
	if (evt != NULL) {
		printf("Rtas event was properly parsed.\n");
		printf("Rtas event:\n");
		servicelog_print_event(stdout, data, 1);
		printf("------------------------------------------------------\n"
				" Rtas event header:\n");
		servicelog_print_header(stdout, data, 0);
	}
	else {
		printf("Rtas event parsing failed\n");
	}

	return 0;
}
